/* callbacks.c                                           */
/*                                                       */
/* This application demonstrates how to do two things.   */
/* The first is how to do context sensitive help using   */
/* your own help button instead of the small help button */
/* that can appear in the window frame.  The second is   */
/* how to cause a balloon to appear as a result of the   */
/* help and have the balloon's color and position be     */
/* widget dependent.                                     */
/*                                                       */
/* For the first thing, the context sensitive help,      */
/* there are basically three tricky parts involved:      */
/* 1. Turning the mouse pointer into a question mark and */
/*    not letting the user do anything else until they   */
/*    click somewhere.  This is done in help_btn_cb()    */
/*    below.                                             */
/* 2. Not letting anything happen while the cursor is a  */
/*    question mark and yet still having the context     */
/*    sensitive help work.  This is solved by blocking   */
/*    the window (see help_btn_cb()) and then getting    */
/*    any user input events by having a Pt_CB_BLOCKED    */
/*    callback on the window (this is                    */
/*    window_blocked_cb().)                              */
/* 3. Finding out what was clicked on.  This is solved   */
/*    in window_blocked_cb() by watching for             */
/*    Ph_EV_BUT_PRESS events.  When we get one we use    */
/*    the mouse position, given in the event, and the    */
/*    PtWidgetHelpHit() function to determine what      */
/*    widget was at the mouse position.                  */
/*                                                       */
/* For the second thing, the balloon, you must:          */
/* 1. Make the balloon appear, done by calling           */
/*    PtInflateBalloon().                                */
/* 2. Make the balloon go away, done using Pt_CB_FILTER  */
/*    and Pt_CB_WINDOW callbacks on the window.          */
/*                                                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.13A */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

/*--------------------------------------------------------------------*
 *
 * help_btn_cb - Pt_CB_ACTIVATE callback attached to the big button
 * with the question mark.
 *
 *--------------------------------------------------------------------*/
int
help_btn_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PtArg_t		arg[3];
	PtWidget_t	*window;
	short		n;
		
	window = widget;
	while ( !PtWidgetIsClass(window, PtWindow) )
		window = PtWidgetParent( window );
	
	n = 0;
	PtSetArg( &arg[n++], Pt_ARG_FLAGS, Pt_BLOCKED, Pt_BLOCKED );
	PtSetArg( &arg[n++], Pt_ARG_CURSOR_TYPE, Ph_CURSOR_QUESTION_POINT, 0 );
	PtSetArg( &arg[n++], Pt_ARG_WINDOW_CURSOR_OVERRIDE, Pt_TRUE, 0 );
	PtSetResources( window, n, arg );
		
	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*--------------------------------------------------------------------*
 *
 * window_blocked_cb - Pt_CB_BLOCKED callback attached to the window.
 *
 * This is called whenever there is user input (mouse stuff, keyboard)
 * for this window and the window is currently blocked (Pt_BLOCKED
 * flag in Pt_ARG_FLAGS.)
 *
 *--------------------------------------------------------------------*/
int
window_blocked_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	char				*topic;
	PhPoint_t			pos;
	PhPointerEvent_t	*pinfo;
	PtArg_t				arg[3];
	PtWidget_t			*help_widget; /* get help for this widget */
    PtWidget_t          *balloon_widget;
	short				x, y, n, *balloon_position;

	if ( cbinfo->event->type != Ph_EV_BUT_PRESS )
		return( Pt_CONTINUE );
		
    /* put things back to normal */
	n = 0;
	PtSetArg( &arg[n++], Pt_ARG_FLAGS, 0, Pt_BLOCKED );
	PtSetArg( &arg[n++], Pt_ARG_CURSOR_TYPE, Ph_CURSOR_POINTER, 0 );
	PtSetArg( &arg[n++], Pt_ARG_WINDOW_CURSOR_OVERRIDE, Pt_FALSE, 0 );
	PtSetResources( widget, n, arg );

	pinfo = PhGetData( cbinfo->event );
	
	/* 'widget' is the window */
	PtGetAbsPosition( widget, &x, &y );
	pos = pinfo->pos;
	pos.x -= x;
	pos.y -= y;
	/* pos is now the coordinate of the ?-mouse click relative to the window */
	/* help_widget will be the widget at the given coordinate that has
	   Pt_ARG_HELP_TOPIC set to something */
	help_widget = PtWidgetHelpHit( widget, &pos );
	if (!help_widget) {
        /* 
         * No widget there with a help topic.  Since we've already put
         * the window state back to normal, we've just cancelled the help.
         */
		return( Pt_CONTINUE );
    }

    /*
     * We've stored the balloon text in the widget's Pt_ARG_HELP_TOPIC
     * resource and the balloon position in the Pt_ARG_BALLOON_POSITION
     * resource.
     */
     
	PtSetArg( &arg[0], Pt_ARG_HELP_TOPIC, &topic, 0 );
    PtSetArg( &arg[1], Pt_ARG_BALLOON_POSITION, &balloon_position, 0 );
	PtGetResources( help_widget, 2, arg );
	
    balloon_widget = PtInflateBalloon( widget, help_widget,
                                       *balloon_position,
                                       topic, "helv14b", Pg_RED, Pg_BLUE );
	
    /*
     * Store the widget pointer to the balloon in the window's
     * Pt_ARG_USER_DATA resource.  This way we don't have to
     * use a global variable and this code becomes reusable for any
     * window.
     */
     
    PtSetArg( &arg[0], Pt_ARG_USER_DATA, &balloon_widget, sizeof(PtWidget_t *) );
    PtSetResources( widget, 1, &arg );
        
	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*--------------------------------------------------------------------*
 *
 * window_filter_cb - Pt_CB_FILTER callback attached to the window.
 *
 * This is called whenever an event arrives at this window and before
 * it is passed onto any children.  It's a handy place for getting
 * rid of the balloon when the user presses a key, clicks on the
 * mouse, etc...
 *
 *--------------------------------------------------------------------*/
int
window_filter_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
    PtArg_t     arg;
    PtWidget_t  **balloon_widget;
    
    PtSetArg( &arg, Pt_ARG_USER_DATA, &balloon_widget, sizeof(PtWidget_t *) );
    PtGetResources( widget, 1, &arg );
    
    if ( balloon_widget && *balloon_widget ) {
        PtDestroyWidget( *balloon_widget );
        *balloon_widget = NULL;
    }

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*--------------------------------------------------------------------*
 *
 * window_cb - Pt_CB_WINDOW callback attached to the window.
 *
 * This is called for the things asked for via that 
 * Pt_ARG_WINDOW_NOTIFY_FLAGS resource.  In this case we've asked to
 * be notify about pretty much anything (window is moved, window gets
 * focus, ...)  It's a handy place for getting rid of the balloon.
 *
 *--------------------------------------------------------------------*/
int
window_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
    PtArg_t     arg;
    PtWidget_t  **balloon_widget;
    
    PtSetArg( &arg, Pt_ARG_USER_DATA, &balloon_widget, sizeof(PtWidget_t *) );
    PtGetResources( widget, 1, &arg );
    
    if ( balloon_widget && *balloon_widget ) {
        PtDestroyWidget( *balloon_widget );
        *balloon_widget = NULL;
    }

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}
